package de.evoal.core.main.properties;

import de.evoal.core.api.properties.Properties;
import de.evoal.core.api.properties.PropertiesSpecification;
import de.evoal.core.api.properties.PropertySpecification;
import de.evoal.core.api.properties.io.PropertiesReader;
import de.evoal.core.api.properties.io.PropertiesWriter;
import de.evoal.core.api.statistics.writer.Column;
import de.evoal.core.api.utils.EvoalIOException;
import lombok.NonNull;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVParser;
import org.apache.commons.csv.CSVPrinter;
import org.apache.commons.csv.CSVRecord;

import javax.enterprise.context.Dependent;
import javax.inject.Named;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.charset.Charset;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Iterator;
import java.util.Locale;

@Slf4j
@Dependent
@Named("csv-writer")
public class CSVPropertiesWriter implements PropertiesWriter {
    private CSVPrinter csvPrinter;

    private FileWriter csvWriter;

    private NumberFormat nf = NumberFormat.getInstance(Locale.US);

    private PropertiesSpecification specification;

    @Override
    public void add(@NonNull Properties properties) throws EvoalIOException {
        try {
            csvPrinter.printRecord(properties.getValues());
        } catch (IOException e) {
            throw new EvoalIOException("Failed to write recored.", e);
        }
    }

    @Override
    public PropertiesWriter init(final File outputFile, final PropertiesSpecification specification) throws EvoalIOException {
        log.info("Creating CSV properties writer for {} to file {}.", specification, outputFile);

        this.specification = specification;

        try {
            final String[] fileHeader = new String[specification.size()];

            for(int i = 0; i < fileHeader.length; ++i) {
                fileHeader[i] = specification.getProperties().get(0).name();
            }

            csvWriter = new FileWriter(outputFile);
            csvPrinter = new CSVPrinter(csvWriter,
                    CSVFormat.DEFAULT
                            .withHeader(fileHeader));
        } catch (IOException e) {
            throw new EvoalIOException("Failed to open CSV file " + outputFile + " for writing.", e);
        }

        return this;
    }

    @Override
    public void close() throws Exception {
        csvWriter.close();
    }
}
